!
! Quick hack to plot the difference between a 2D real field
! in a WRF binary files.
!

program dplot

        use module_io_int_idx, only: io_int_index, r_info
        use mpiio_rw, only: read_data
        use mpi

        implicit none

        integer, parameter :: llong_t  = selected_int_kind(16)  ! int64_t

        integer                            :: argc
        character(len=256)                 :: argv

        integer                            :: ierr
        integer                            :: i,j,k
        integer                            :: iunit_1
        integer                            :: iunit_2
        integer(kind=llong_t)              :: offset
        integer                            :: count
        character(len=256)                 :: filename
        character(len=256)                 :: file_1
        character(len=256)                 :: file_2
        type(r_info), pointer              :: idx1(:) => NULL()
        type(r_info), pointer              :: idx2(:) => NULL()
        integer                            :: nrecords
        character(len=256)                 :: var
        real, dimension(:,:,:), allocatable:: r1_3d
        real, dimension(:,:,:), allocatable:: r2_3d
        real, dimension(:,:,:), allocatable:: d_3d
        real, dimension(:,:), allocatable  :: r1_2d
        real, dimension(:,:), allocatable  :: r2_2d
        real, dimension(:,:), allocatable  :: d_2d
        integer                            :: dims(3)

        namelist /plot/ file_1, file_2, var, dims, i, j, k


        argc     = 0
        filename = ''
        ierr     = 0
        offset   = 0
        count    = 0
        nrecords = 0
        var      = ''
        dims     = (/ 1, 1, 0 /)
        i        = 1
        j        = 1
        k        = 1

        call mpi_init(ierr)
        ! Get the namelist file name
        argc = iargc()
        if (argc .ne. 1 ) then
                write(0, *) 'Need 1 argument, namelist filename'
                write(0, *) 'e.g.: bplot plot.nl'
                call exit(1)
        endif
        call getarg(1, filename)
        iunit_1 = 7
        iunit_2 = 8
        open(iunit_1, file=trim(filename), iostat=ierr)
        if (ierr .ne. 0) then
                write(0,*) 'Unable to open namelist file: ', trim(filename)
                call exit(ierr)
        endif

        read(iunit_1, plot)
        close(iunit_1)

        ! Get the file index and open the files
        call io_int_index(file_1, idx1, ierr)
        if (ierr .ne. 0) then
                call exit(ierr)
        endif
        call io_int_index(file_2, idx2, ierr)
        if (ierr .ne. 0) then
                call exit(ierr)
        endif

        call mpi_file_open(mpi_comm_world, trim(file_1), &
                           mpi_mode_rdonly, mpi_info_null, &
                           iunit_1, ierr)
        if (ierr .ne. 0) then
                write(0,*) 'Unable to open:', trim(file_1)
                call exit(2)
        end if
        call mpi_file_open(mpi_comm_world, trim(file_2), &
                           mpi_mode_rdonly, mpi_info_null, &
                           iunit_2, ierr)
        if (ierr .ne. 0) then
                write(0,*) 'Unable to open:', trim(file_2)
                call exit(2)
        end if

        select case(dims(3))
        case(0)
                allocate(r1_2d(dims(1),dims(2)))
                allocate(r2_2d(dims(1),dims(2)))
                allocate(d_2d(dims(1),dims(2)))
                r1_2d = 0.0
                r2_2d = 0.0
                d_2d  = 0.0
                call read_data(ifd=iunit_1, records=idx1, varname=trim(var), &
                               dst=r1_2d, ierr=ierr)
                call read_data(ifd=iunit_2, records=idx2, varname=trim(var), &
                               dst=r2_2d, ierr=ierr)
                d_2d = r1_2d - r2_2d
                call plot_data(trim(var), d_2d)
        case default
                allocate(r1_3d(dims(1),dims(2),dims(3)))
                allocate(r2_3d(dims(1),dims(2),dims(3)))
                allocate(d_3d(dims(1),dims(2),dims(3)))
                r1_3d = 0.0
                r2_3d = 0.0
                d_3d  = 0.0
                call read_data(ifd=iunit_1, records=idx1, varname=trim(var), &
                               dst=r1_3d, ierr=ierr)
                call read_data(ifd=iunit_2, records=idx2, varname=trim(var), &
                               dst=r2_3d, ierr=ierr)
                d_3d = r1_3d - r2_3d
                call plot_data(trim(var), d_3d(:,:,k))
        end select
        close(iunit_1)
        close(iunit_2)


        ! Free everything and get out
        if (allocated(r1_3d)) then
                deallocate(r1_3d)
        end if
        if (allocated(r1_2d)) then
                deallocate(r1_2d)
        end if
        if (allocated(r2_3d)) then
                deallocate(r2_3d)
        end if
        if (allocated(r2_2d)) then
                deallocate(r2_2d)
        end if
        if (allocated(d_2d)) then
                deallocate(d_2d)
        end if
        if (allocated(d_3d)) then
                deallocate(d_3d)
        end if

        call mpi_finalize(ierr)

        contains

subroutine plot_data(v, d)
        use plplot
        implicit none

        character(len=*),     intent(in)    :: v
        real,                 intent(inout) :: d(:,:)

        integer, parameter :: NLEVEL = 40

        integer :: i, j
        real :: shedge(NLEVEL+1)
        character*1 :: defined
        real :: xmin, xmax, ymin, ymax, zmin, zmax
        character(len=1024) :: filename

        xmin = lbound(d,1)
        ymin = lbound(d,2)
        xmax = ubound(d,1)
        ymax = ubound(d,2)
        zmin = minval(d) - 0.01
        zmax = maxval(d) + 0.01
        shedge = 0.0

        do i = 1, NLEVEL+1
                shedge(i) = zmin + (zmax - zmin)/NLEVEL*(i-1)
        enddo

        filename = trim(v)//'.pdf'
        call plsfnam(trim(filename))
        call plstart("pdf", 1, 1)
        call plenv(xmin, xmax, ymin, ymax, 0, 0)
        call plspal1("cmap1_blue_red.pal", 1)

        call plshades(d, defined, xmin, xmax, ymin, ymax, shedge, 2, 0, 0)
        call pllab('X', 'Y', trim(v))
        call plend()

end subroutine plot_data

end program dplot

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Stub subroutines (so we don't have to build against the whole of WRF !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine wrf_message(message)
        implicit none

        character(len=*), intent(in) :: message

        write(0,*) trim(message)
end subroutine wrf_message

subroutine wrf_error_fatal3(file, line, message)
        implicit none

        character(len=*), intent(in) :: file
        integer,          intent(in) :: line
        character(len=*), intent(in) :: message

        write(0,*) trim(file), 'line: ', line, ': ', trim(message)
        stop(1)
end subroutine wrf_error_fatal3

subroutine wrf_debug(n, message)
        integer,          intent(in) :: n
        character(len=*), intent(in) :: message

        write(0,*) trim(message)
end subroutine wrf_debug
